# frozen_string_literal: true

def add_bury_to_hash
  Hash.send(:define_method, :bury) do |*args|
    if args.count < 2
      raise ArgumentError, '2 or more arguments required'
    elsif args.count == 2
      self[args[0]] = args[1]
    else
      arg = args.shift
      self[arg] = {} unless self[arg]
      self[arg].bury(*args) unless args.empty?
    end
  end
end

class AbstractTranslation
  attr_accessor :template,
                :context,
                :data,
                :ancestors

  def initialize
    add_bury_to_hash

    @ancestors  = []
    @template   = {}
    @context    = nil
    @data       = {}
  end

  def inspect
    @template
  end
end

class AbstractTranslator
  def abstract_method_missing(translation, method_name, keys, *args)
    if translation.context.nil?
      unless keys.include?(method_name.to_sym) ||
             keys.include?(method_name.to_s)
        raise ArgumentError.new(
          method_name: method_name,
          keys: keys
        )
      end
    end

    keys.each do |key|
      case translation.context
      when key
        translation.ancestors.push(method_name)
        yield if block_given?
        if args.length == 1
          translation.template.bury(*translation.ancestors, args[0])
          translation.ancestors.pop
        elsif args.empty?
          translation.ancestors.pop
        else
          raise ArgumentError,
                "field: #{
                      method_name
                    } must have exactly one argument, had #{
                        args.length
                      }, which were #{args}"
        end
      end
      next unless [key.to_s, key.to_sym].include?(method_name)

      translation.ancestors.push(method_name.to_sym)
      translation.ancestors.push(*args)
      translation.context = method_name.to_sym
      yield if block_given?
      translation.ancestors = []
      translation.context = nil
    end
  end

  def eval_in_context(content)
    instance_eval(content)
  end
end
