# frozen_string_literal: true

###############################################################################
# define a run interface that takes a dict
# then dispatch accordingly
###############################################################################

require 'pleme/plemec_translator'
require 'yaml'
require 'json'
require 'pleme/scanner'

class Runner
  def initialize
    @config_path  = 'plemec.rb'
    @translator   = PlemecTranslator.new
    @spec         = nil
  end

  # run a shell command
  # hard exit if command has a failure
  # like
  # set -e
  def sh(cmd)
    exit 1 unless system(cmd)
  end

  # pipe hash to yaml without
  # weird symbol issues
  def yamilify(rhash)
    JSON.parse(rhash.to_json).to_yaml
  end

  def run(spec)
    $logger.info spec

    @spec = spec    if @spec.nil?
    dotfiles(spec)  if spec[:dotfiles]

    compose if spec[:compose]

    if spec[:truvantis] && spec[:scan]
      scan = Scanner.new.truvantis_public_scan
      File.write('truvantis_scan.json', JSON.pretty_generate(scan))
    end
  end

  def read_config
    @translator.eval_in_context(File.read(@config_path))
    @translator.translation.inspect
  end

  def dotfiles(_spec)
    hax_portal
  end

  def compose
    config = read_config
    $logger.info config
    config[:pleme][:environment].keys.each do |env|
      config[:pleme][:environment][env].keys.each do |param|
        case param
        when :compose
          File.write('docker-compose.yml', yamilify(config[:pleme][:environment][env][:compose]))
          sh "docker-compose #{ARGV[1..-1].join(' ')}"
        end
      end
    end
  end

  # read a local file into this space
  # so you can test locally while importing
  # good code to this class
  def hax_portal
    instance_eval(File.read('hax.rb')) if File.exist?('hax.rb')
  end
end
